"use strict";

exports.__esModule = true;

var _promise = require("babel-runtime/core-js/promise");

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require("babel-runtime/regenerator");

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require("babel-runtime/helpers/asyncToGenerator");

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require("babel-runtime/helpers/classCallCheck");

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _possibleConstructorReturn2 = require("babel-runtime/helpers/possibleConstructorReturn");

var _possibleConstructorReturn3 = _interopRequireDefault(_possibleConstructorReturn2);

var _inherits2 = require("babel-runtime/helpers/inherits");

var _inherits3 = _interopRequireDefault(_inherits2);

var _map = require("babel-runtime/core-js/map");

var _map2 = _interopRequireDefault(_map);

var _stcPlugin = require("stc-plugin");

var _stcPlugin2 = _interopRequireDefault(_stcPlugin);

var _helper = require("./helper");

var _stcHelper = require("stc-helper");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var REG = {
  ID: /id\s*=\s*['"]([^'">]+)['"]/,
  JS_ID: /([W$]\(['"]#([^'"]+)['"]\)\.html\(\))/
};
var tplMap = new _map2.default();
var sourceTplMap = new _map2.default();

var jsTplReplace = function (_Plugin) {
  (0, _inherits3.default)(jsTplReplace, _Plugin);

  function jsTplReplace() {
    (0, _classCallCheck3.default)(this, jsTplReplace);
    return (0, _possibleConstructorReturn3.default)(this, _Plugin.apply(this, arguments));
  }

  /** 
   * collect script tokens whose `type` is `text/html` or `text/template` by default,
   * put their content into a global map
   * we will use it later
  **/

  jsTplReplace.prototype.run = function () {
    var _ref = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee() {
      return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              if (!this.isTpl()) {
                _context.next = 2;
                break;
              }

              return _context.abrupt("return", this.parseJsTplTags());

            case 2:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));

    function run() {
      return _ref.apply(this, arguments);
    }

    return run;
  }();

  jsTplReplace.prototype.parseJsTplTags = function () {
    var _ref2 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee2() {
      var _this2 = this;

      var tokens, promises;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              _context2.next = 2;
              return this.getAst();

            case 2:
              tokens = _context2.sent;
              promises = tokens.map(function (token) {
                var start = token.ext.start;
                var content = token.ext.content;
                if (token.type === _this2.TokenType.HTML_TAG_SCRIPT && start.ext.isTpl && !start.ext.isExternal) {
                  var val = start.value;
                  var matchResult = val.match(REG.ID);
                  if (matchResult && matchResult[1] && !content.ext.hasTpl) {
                    var id = matchResult[1].trim();

                    if (tplMap.has(id)) {
                      var existedMd5 = (0, _stcHelper.md5)(tplMap.get(id).content.trim());
                      var curMd5 = (0, _stcHelper.md5)(content.value.trim());
                      if (curMd5 !== existedMd5) {
                        _this2.fatal('you have duplicate id: ' + id);
                        return;
                      }
                      tplMap.get(id)['files'].push(_this2.file);
                    } else {
                      //get the content, remove the source tpl and add it to the map
                      tplMap.set(id, {
                        files: [_this2.file],
                        content: content.value
                      });
                    }
                  }
                }
              });
              _context2.next = 6;
              return _promise2.default.all(promises);

            case 6:
            case "end":
              return _context2.stop();
          }
        }
      }, _callee2, this);
    }));

    function parseJsTplTags() {
      return _ref2.apply(this, arguments);
    }

    return parseJsTplTags;
  }();
  /**
   * remove used script token in tpl files
  **/


  jsTplReplace.prototype.removeJsTplTags = function () {
    var _ref3 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee3(file, id) {
      var tokens, index, curToken, start, content, val, matchResult, curId;
      return _regenerator2.default.wrap(function _callee3$(_context3) {
        while (1) {
          switch (_context3.prev = _context3.next) {
            case 0:
              _context3.next = 2;
              return file.getAst();

            case 2:
              tokens = _context3.sent;


              for (index in tokens) {
                curToken = tokens[index];
                start = curToken.ext.start;
                content = curToken.ext.content;
                //比较id

                if (curToken.type === this.TokenType.HTML_TAG_SCRIPT && start.ext.isTpl && !start.ext.isExternal) {
                  val = start.value;
                  matchResult = val.match(REG.ID);

                  if (matchResult && matchResult[1] && !content.ext.hasTpl) {
                    curId = matchResult[1].trim();

                    if (curId === id) {
                      tokens.splice(index, 1);
                    }
                  }
                }
              }
              file.setAst(tokens);

            case 5:
            case "end":
              return _context3.stop();
          }
        }
      }, _callee3, this);
    }));

    function removeJsTplTags(_x, _x2) {
      return _ref3.apply(this, arguments);
    }

    return removeJsTplTags;
  }();

  jsTplReplace.prototype.update = function update(data) {
    if (!data) return;
    if (data.error) {
      this.error(data.error, data.line, data.column);
    }
  };
  /**
   * default include
   */


  jsTplReplace.include = function include() {
    return [/\.js$/, { type: 'tpl' }];
  };
  /**
   * after handling templates files, 
   * we need to replace strings like `$('#myTplId').html` in js with content collected before,
   * and remove those content in template files.
  **/


  jsTplReplace.after = function () {
    var _ref4 = (0, _asyncToGenerator3.default)(_regenerator2.default.mark(function _callee4(files, instance) {
      var index, file, content, arr, _index, match, id, tplObj, jsTplStr, tplVarRegExp, tplFiles, index1, tplFile;

      return _regenerator2.default.wrap(function _callee4$(_context4) {
        while (1) {
          switch (_context4.prev = _context4.next) {
            case 0:
              _context4.t0 = _regenerator2.default.keys(files);

            case 1:
              if ((_context4.t1 = _context4.t0()).done) {
                _context4.next = 12;
                break;
              }

              index = _context4.t1.value;
              file = files[index];
              _context4.next = 6;
              return file.getContent('utf8');

            case 6:
              content = _context4.sent;
              arr = (0, _helper.matchAll)(content, REG.JS_ID);


              if (arr.length) {
                for (_index in arr) {
                  match = arr[_index];


                  if (match && match[2]) {
                    id = match[2].trim();
                    tplObj = tplMap.get(id);

                    if (tplMap.has(id)) {
                      jsTplStr = tplObj.content;
                      tplVarRegExp = '[W$]\\([\'"]#' + id + '[\'"]\\)\\.html\\(\\)';

                      tplVarRegExp = new RegExp(tplVarRegExp, 'g');

                      jsTplStr = jsTplStr.replace(/\n/g, '\\n');
                      jsTplStr = jsTplStr.replace(/"/g, '\\"');

                      content = content.replace(tplVarRegExp, '"' + jsTplStr + '"');
                      // remove the content in tpl file
                      tplFiles = tplObj.files;

                      for (index1 in tplFiles) {
                        tplFile = tplFiles[index1];

                        instance.removeJsTplTags(tplFile, id);
                      }
                    }
                  }
                }
              }
              file.setContent(content);
              _context4.next = 1;
              break;

            case 12:
            case "end":
              return _context4.stop();
          }
        }
      }, _callee4, this);
    }));

    function after(_x3, _x4) {
      return _ref4.apply(this, arguments);
    }

    return after;
  }();

  return jsTplReplace;
}(_stcPlugin2.default);

exports.default = jsTplReplace;
//# sourceMappingURL=/Users/huangxiaolu/Documents/webtasks/lab/stcjs/stc-js-tpl-replace/lib/index.js.map